@@ -23,6 +23,8 @@
 #ifndef UI_H
 #define UI_H
 
+#include <stdint.h>
+
 #include "calctool.h"
 
 void ui_init(int *argc, char ***argv);
@@ -33,14 +35,13 @@ void ui_make_registers(void);
 void ui_set_undo_enabled(gboolean, gboolean);
 
 void ui_set_display(char *, int);
+void ui_set_bitfield(int enabled, guint64 bits);
 gchar *ui_get_display(void);
 
 void ui_set_registers_visible(gboolean);
 void ui_set_accuracy(int);
 void ui_set_mode(enum mode_type);
 void ui_set_base(enum base_type);
-void ui_set_inverse_state(gboolean);
-void ui_set_hyperbolic_state(gboolean);
 void ui_set_trigonometric_mode(enum trig_type);
 void ui_set_numeric_mode(enum base_type);
 void ui_set_show_thousands_separator(gboolean);
